'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var funcadelic = require('funcadelic');
var SymbolObservable = _interopDefault(require('symbol-observable'));

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _toArray(arr) {
  return _arrayWithHoles(arr) || _iterableToArray(arr) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _iterableToArrayLimit(arr, i) {
  if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) {
    return;
  }

  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function CachedProperty(key, reify) {
  let enumerable = true;
  let configurable = true;
  return {
    enumerable,
    configurable,

    get() {
      let value = reify(this);
      Object.defineProperty(this, key, {
        enumerable,
        value
      });
      return value;
    }

  };
}

var _class, _temp;
const Tree = funcadelic.type((_temp = _class = class {
  childAt(key, parent) {
    if (parent[Tree.symbol]) {
      return this(parent).childAt(key, parent);
    } else {
      return parent[key];
    }
  }

  defineChildren(fn, parent) {
    if (parent[Tree.symbol]) {
      return this(parent).defineChildren(fn, parent);
    } else {
      for (var _i = 0, _Object$keys = Object.keys(parent); _i < _Object$keys.length; _i++) {
        let property = _Object$keys[_i];
        Object.defineProperty(parent, property, CachedProperty(property, () => fn(property, parent)));
      }
    }
  }

}, _defineProperty(_class, "name", 'Tree'), _temp));
const _Tree$prototype = Tree.prototype,
      childAt = _Tree$prototype.childAt,
      defineChildren = _Tree$prototype.defineChildren;

class Box {
  static get of() {
    return (...args) => new this(...args);
  }

  static unwrap(box) {
    return box.value;
  }

  constructor(value) {
    this.value = value;
  }

}

const unwrap = Box.unwrap;

class Id extends Box {}

class Const extends Box {}

funcadelic.Functor.instance(Id, {
  map(fn, id) {
    return Id.of(fn(id.value));
  }

});
funcadelic.Functor.instance(Const, {
  map(fn, constant) {
    return constant;
  }

});
function compose(f, g) {
  return (...x) => f(g(...x));
}
function view(lens, context) {
  let get = compose(unwrap, lens(Const.of));
  return get(context);
}
function over(lens, fn, context) {
  let update = compose(unwrap, lens(compose(Id.of, fn)));
  return update(context);
}
function set(lens, value, context) {
  return over(lens, () => value, context);
}
function Lens(get, set) {
  return f => context => {
    return funcadelic.map(value => set(value, context), f(get(context)));
  };
}
const transparent = Lens(x => x, y => y);
function At(property, container) {
  let get = context => context != null ? childAt(property, context) : undefined;

  let set = (part, whole) => {
    let context = whole == null ? Array.isArray(container) ? [] : {} : whole;

    if (part === context[property]) {
      return context;
    } else if (Array.isArray(context)) {
      let clone = context.slice();
      clone[Number(property)] = part;
      return clone;
    } else {
      return funcadelic.Semigroup.for(Object).append(context, {
        [property]: part
      });
    }
  };

  return Lens(get, set);
}
function Path(path) {
  return path.reduce((lens, key) => compose(lens, At(key)), transparent);
}

class Meta {
  constructor(object, value) {
    this.root = object;
    this.lens = transparent;
    this.path = [];
    this.value = value;
    this.source = object;
  }

}

_defineProperty(Meta, "symbol", Symbol('Meta'));

_defineProperty(Meta, "data", At(Meta.symbol));

_defineProperty(Meta, "lens", compose(Meta.data, At('lens')));

_defineProperty(Meta, "path", compose(Meta.data, At('path')));

_defineProperty(Meta, "value", compose(Meta.data, At('value')));

_defineProperty(Meta, "source", compose(Meta.data, At('source')));

function metaOf(object) {
  return view(Meta.data, object);
}
function valueOf(object) {
  let meta = metaOf(object);
  return meta != null ? meta.value : object;
}
function sourceOf(object) {
  return view(Meta.source, object);
}
function mount(microstate, substate, key) {
  let parent = view(Meta.data, microstate);
  let prefix = compose(parent.lens, At(key, parent.value));
  return over(Meta.data, meta => ({
    get root() {
      return parent.root;
    },

    get lens() {
      return compose(prefix, meta.lens);
    },

    get path() {
      return parent.path.concat([key]).concat(meta.path);
    },

    get value() {
      return meta.value;
    },

    get source() {
      return meta.source;
    }

  }), substate);
}

const getPrototypeOf = Object.getPrototypeOf,
      getOwnPropertyDescriptors = Object.getOwnPropertyDescriptors,
      assign = Object.assign;
function methodsOf(Type) {
  return funcadelic.filter(({
    key: name,
    value: desc
  }) => {
    return name !== 'constructor' && typeof name === 'string' && typeof desc.value === 'function';
  }, getAllPropertyDescriptors(Type.prototype));
}

function getAllPropertyDescriptors(object) {
  if (object === Object.prototype) {
    return {};
  } else {
    let prototype = getPrototypeOf(object);
    return assign(getAllPropertyDescriptors(prototype), getOwnPropertyDescriptors(object));
  }
}

class Primitive {
  get state() {
    return valueOf(this);
  }

}

class Any extends Primitive {}

function query(iterable) {
  return new Query(iterable);
}
function map(iterable, fn) {
  return query(iterable).map(fn);
}
function filter(iterable, fn) {
  return query(iterable).filter(fn);
}
function reduce(iterable, fn, initial) {
  return query(iterable).reduce(fn, initial);
}
function find(iterable, fn) {
  return query(iterable).find(fn);
}

class Query {
  constructor(iterable) {
    if (typeof iterable[Symbol.iterator] === 'function') {
      this.generator = () => iterable[Symbol.iterator]();
    } else if (typeof iterable === 'function') {
      this.generator = iterable;
    } else {
      throw new Error('Query must be constructed with a generator function or iterable. Received `${iterable}`');
    }
  }

  get length() {
    return reduce(this, sum => sum + 1, 0);
  }

  map(fn) {
    return query(() => {
      let source = this.generator();
      return {
        next() {
          let next = source.next();
          return {
            get done() {
              return next.done;
            },

            get value() {
              return fn(next.value);
            }

          };
        }

      };
    });
  }

  filter(fn) {
    return query(() => {
      let source = this.generator();
      return {
        next() {
          let result;

          function find() {
            if (result != null) {
              return result;
            } else {
              for (result = source.next(); !result.done && !fn(result.value); result = source.next()) {}

              return result;
            }
          }

          return {
            get done() {
              return find().done;
            },

            get value() {
              return find().value;
            }

          };
        }

      };
    });
  }

  reduce(fn, initial) {
    let result = initial;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = this[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        let item = _step.value;
        result = fn(result, item);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return result;
  }

  find(fn) {
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = this[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        let item = _step2.value;

        if (fn(item)) {
          return item;
        }
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }
  }

  [Symbol.iterator]() {
    return this.generator();
  }

}

function parameterized(fn) {
  function initialize(...args) {
    let Type = fn(...args);

    if (Type.initialize) {
      Type.initialize();
    }

    return Type;
  }

  let defaultTypeParameters = new Array(fn.length);
  defaultTypeParameters.fill(Any);
  let DefaultType = initialize(...defaultTypeParameters);

  DefaultType.of = (...args) => initialize(...args);

  return DefaultType;
}

var ObjectType = parameterized(T => {
  var _class, _temp;

  let _Symbol$iterator;

  return _temp = (_Symbol$iterator = Symbol.iterator, _class = class ObjectType {
    get entries() {
      return reduce(this, (entries, entry) => Object.assign(entries, {
        [entry.key]: entry.value
      }), {});
    }

    get keys() {
      return Object.keys(valueOf(this));
    }

    get values() {
      return query(this).map(entry => entry.value);
    }

    initialize(value) {
      return value == null ? {} : this;
    }

    assign(attrs) {
      return funcadelic.append(valueOf(this), funcadelic.map(valueOf, attrs));
    }

    put(name, value) {
      return this.assign({
        [name]: value
      });
    }

    delete(name) {
      return funcadelic.filter(({
        key
      }) => key !== name, valueOf(this));
    }

    map(fn) {
      return funcadelic.map(value => valueOf(fn(create(T, value))), valueOf(this));
    }

    filter(fn) {
      return funcadelic.filter(({
        key,
        value
      }) => valueOf(fn(create(T, value), key)), valueOf(this));
    }

    [_Symbol$iterator]() {
      let object = this;
      let iterator = Object.keys(valueOf(this))[Symbol.iterator]();
      return {
        next() {
          let next = iterator.next();
          return {
            get done() {
              return next.done;
            },

            get value() {
              if (!next.done) {
                return new Entry(next.value, childAt(next.value, object));
              } else {
                return undefined;
              }
            }

          };
        }

      };
    }

    static initialize() {
      Tree.instance(this, {
        childAt(key, object) {
          if (typeof key !== 'string') {
            return object[key];
          } else {
            let value = valueOf(object)[key];
            return mount(object, create(T, value), key);
          }
        },

        defineChildren(fn, object) {
          let generate = object[Symbol.iterator];
          return Object.defineProperty(object, Symbol.iterator, {
            enumerable: false,

            value() {
              let iterator = generate.call(object);
              return {
                next() {
                  let next = iterator.next();
                  return {
                    get done() {
                      return next.done;
                    },

                    get value() {
                      if (!next.done) {
                        let key = next.value.key;
                        return new Entry(key, fn(key));
                      } else {
                        return undefined;
                      }
                    }

                  };
                }

              };
            }

          });
        }

      });
    }

  }), _defineProperty(_class, "T", T), _defineProperty(_class, "name", `Object<${T.name}>`), _temp;
});

class Entry {
  constructor(key, value) {
    this.key = key;
    this.value = value;
  }

  [Symbol.iterator]() {
    return [this.value, this.key][Symbol.iterator]();
  }

}

var ArrayType = parameterized(T => {
  var _class, _temp;

  let _Symbol$iterator;

  return _temp = (_Symbol$iterator = Symbol.iterator, _class = class ArrayType {
    static get name() {
      return `Array<${T.name}>`;
    }

    get length() {
      return valueOf(this).length;
    }

    initialize(value) {
      if (value == null) {
        return [];
      } else if (Array.isArray(value) || value[Symbol.iterator]) {
        return value;
      } else {
        return [value];
      }
    }

    push(value) {
      return [...valueOf(this), valueOf(value)];
    }

    pop() {
      return valueOf(this).slice(0, -1);
    }

    shift() {
      let _valueOf = valueOf(this),
          _valueOf2 = _toArray(_valueOf),
          rest = _valueOf2.slice(1);

      return rest;
    }

    unshift(value) {
      return [valueOf(value), ...valueOf(this)];
    }

    slice(begin, end) {
      let list = valueOf(this);
      let result = list.slice(begin, end);
      return list.length === result.length ? this : result;
    }

    sort(compareFn) {
      let init = valueOf(this);
      let result = [...init].sort(compareFn);
      return init.every((member, idx) => result[idx] === member) ? this : result;
    }

    filter(fn) {
      let list = valueOf(this);
      let result = list.filter(member => fn(create(T, member)));
      return list.length === result.length ? this : result;
    }

    map(fn) {
      let list = valueOf(this);
      return list.reduce((result, member, index) => {
        let mapped = valueOf(fn(create(T, member)));
        return set(At(index, result), mapped, result);
      }, list);
    }

    remove(item) {
      return this.filter(s => valueOf(s) !== valueOf(item));
    }

    clear() {
      return [];
    }

    [_Symbol$iterator]() {
      let array = this;
      let iterator = valueOf(this)[Symbol.iterator]();
      let i = 0;
      return {
        next() {
          let next = iterator.next();
          let index = i++;
          return {
            get done() {
              return next.done;
            },

            get value() {
              if (!next.done) {
                return childAt(index, array);
              } else {
                return undefined;
              }
            }

          };
        }

      };
    }

    static initialize() {
      Tree.instance(this, {
        childAt(key, array) {
          if (typeof key === 'number') {
            let value = valueOf(array)[key];
            return mount(array, create(T, value), key);
          } else {
            return array[key];
          }
        },

        defineChildren(fn, array) {
          let generate = array[Symbol.iterator];
          return Object.defineProperty(array, Symbol.iterator, {
            enumerable: false,

            value() {
              let iterator = generate.call(array);
              let i = 0;
              return {
                next() {
                  let next = iterator.next();
                  let index = i++;
                  return {
                    get done() {
                      return next.done;
                    },

                    get value() {
                      if (!next.done) {
                        return fn(index, next.value, array);
                      } else {
                        return undefined;
                      }
                    }

                  };
                }

              };
            }

          });
        }

      });
    }

  }), _defineProperty(_class, "T", T), _temp;
});

class BooleanType extends Primitive {
  initialize(value) {
    return !!value;
  }

  toggle() {
    return !this.state;
  }

}

_defineProperty(BooleanType, "name", "Boolean");

class NumberType extends Primitive {
  initialize(value) {
    if (value == null) {
      return 0;
    } else if (isNaN(value)) {
      return this;
    } else {
      return Number(value);
    }
  }

  increment(step = 1) {
    return this.state + step;
  }

  decrement(step = 1) {
    return this.state - step;
  }

}

_defineProperty(NumberType, "name", "Number");

class StringType extends Primitive {
  initialize(value) {
    if (value == null) {
      return '';
    } else {
      return String(value);
    }
  }

  concat(value) {
    return this.state.concat(value);
  }

}

_defineProperty(StringType, "name", "String");

class DSL {
  constructor(rules = []) {
    this.rules = rules;
  }

  expand(value) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = this.rules[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        let rule = _step.value;
        let result = rule.call(this, value);

        if (result && typeof result.Type === 'function') {
          return result;
        } else {
          continue;
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return {
      Type: Any,
      value
    };
  }

  use(rule) {
    return new DSL(this.rules.concat(rule));
  }

}

var dsl = new DSL().use(function matchBuiltins(value) {
  switch (value) {
    case Object:
      return {
        Type: ObjectType,
        value: {}
      };

    case Array:
      return {
        Type: ArrayType,
        value: []
      };

    case Boolean:
      return {
        Type: BooleanType,
        value: false
      };

    case Number:
      return {
        Type: NumberType,
        value: 0
      };
  }

  if (value === String) {
    return {
      Type: StringType,
      value: ''
    };
  }
}).use(function matchArrays(value) {
  if (Array.isArray(value) && value.length < 2) {
    let _value = _slicedToArray(value, 1),
        T = _value[0];

    let Type = T != null ? ArrayType.of(this.expand(T).Type) : ArrayType;
    return {
      Type,
      value: []
    };
  }
}).use(function matchObjects(value) {
  if (value != null && typeof value === 'object' && Object.keys(value).length < 2) {
    let _Object$keys = Object.keys(value),
        _Object$keys2 = _slicedToArray(_Object$keys, 1),
        key = _Object$keys2[0];

    if (key == null) {
      return {
        Type: ObjectType,
        value: {}
      };
    } else {
      let _this$expand = this.expand(value[key]),
          childType = _this$expand.Type;

      if (childType.isConstant) {
        return undefined;
      } else {
        return {
          Type: ObjectType.of(childType),
          value: {}
        };
      }
    }
  }
}).use(function matchCustomTypes(value) {
  if (typeof value === 'function') {
    return {
      Type: value,
      value: undefined
    };
  }
}).use(function matchConstants(value) {
  return {
    Type: Constant(),
    value
  };
});

const Constant = () => {
  var _class, _temp;

  return _temp = _class = class Constant extends Primitive {}, _defineProperty(_class, "isConstant", true), _temp;
};

class Relationship {
  constructor(traverse) {
    this.traverse = traverse;
  }

  flatMap(sequence) {
    return new Relationship(edge => {
      let cell = this.traverse(edge);
      let next = sequence(cell);
      return next.traverse(edge);
    });
  }

  map(fn) {
    return this.flatMap(cell => new Relationship(() => fn(cell)));
  }

}
function relationship(definition) {
  if (typeof definition === 'function') {
    return new Relationship(definition);
  } else {
    return relationship(({
      value
    }) => {
      if (value != null) {
        return {
          Type: Any,
          value
        };
      } else {
        return {
          Type: Any,
          value: definition
        };
      }
    });
  }
}
class Edge {
  constructor(parent, path) {
    this.parent = parent;
    this.path = path;
  }

  get name() {
    let _this$path$slice = this.path.slice(-1),
        _this$path$slice2 = _slicedToArray(_this$path$slice, 1),
        name = _this$path$slice2[0];

    return name;
  }

  get value() {
    return view(Path(this.path), this.parentValue);
  }

  get parentValue() {
    return valueOf(this.parent);
  }

}

class Storage {
  constructor(value, observe = x => x) {
    this.value = value;
    this.observe = observe;
  }

  get() {
    return this.value;
  }

  set(value) {
    if (value !== this.value) {
      this.value = value;
      this.observe();
    }

    return this;
  }

  getPath(path) {
    return view(Path(path), this.value);
  }

  setPath(path, value) {
    return this.set(set(Path(path), value, this.value));
  }

}

function Pathmap(Root, ref) {
  let paths = new Storage();

  class Location {
    static allocate(path) {
      let existing = paths.getPath(path.concat(Location.symbol));

      if (existing) {
        return existing;
      } else {
        let location = new Location(path);
        paths.setPath(path, {
          [Location.symbol]: location
        });
        return location;
      }
    }

    get currentValue() {
      return valueOf(this.microstate);
    }

    get reference() {
      if (!this.currentReference || this.currentValue !== valueOf(this.currentReference)) {
        return this.currentReference = this.createReference();
      } else {
        return this.currentReference;
      }
    }

    get microstate() {
      return view(this.lens, create(Root, ref.get()));
    }

    constructor(path) {
      this.path = path;
      this.lens = Path(path);
      this.createReferenceType = funcadelic.stable(Type => {
        let location = this;
        let typeName = Type.name ? Type.name : 'Unknown';

        class Reference extends Type {
          constructor(value) {
            super(value);
            Object.defineProperty(this, Meta.symbol, {
              enumerable: false,
              configurable: true,
              value: new Meta(this, valueOf(value))
            });
            defineChildren(key => Location.allocate(path.concat(key)).reference, this);
          }

        }

        _defineProperty(Reference, "name", `Ref<${typeName}>`);

        _defineProperty(Reference, "location", location);

        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = Object.keys(methodsOf(Type)).concat("set")[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            let methodName = _step.value;

            Reference.prototype[methodName] = (...args) => {
              let microstate = location.microstate;
              let next = microstate[methodName](...args);
              ref.set(valueOf(next));
              return location.reference;
            };
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        return Reference;
      });
    }

    createReference() {
      let Type = this.microstate.constructor.Type;
      let Reference = this.createReferenceType(Type);
      return new Reference(this.currentValue);
    }

    get(reference = paths.getPath([Location.symbol, 'reference'])) {
      return reference.constructor.location.reference;
    }

  }

  _defineProperty(Location, "symbol", Symbol('Location'));

  return Location.allocate([]);
}

function Identity(microstate, observe = x => x) {
  let Type = microstate.constructor.Type;
  let pathmap = Pathmap(Type, new Storage(valueOf(microstate), () => observe(pathmap.get())));
  return pathmap.get();
}

function Observable(Microstate) {
  return class extends Microstate {
    [SymbolObservable]() {
      return this['@@observable']();
    }

    ['@@observable']() {
      return {
        subscribe: observer => {
          let next = observer.call ? observer : observer.next.bind(observer);
          let identity = Identity(this, next);
          next(identity);
          return identity;
        },

        [SymbolObservable]() {
          return this;
        }

      };
    }

  };
}

function create(InputType = Any, value) {
  let _dsl$expand = dsl.expand(InputType),
      Type = _dsl$expand.Type;

  let Microstate = MicrostateType(Type);
  let microstate = new Microstate(value);

  if (hasOwnProperty(Type.prototype, 'initialize')) {
    return microstate.initialize(value);
  } else {
    return microstate;
  }
}
const MicrostateType = funcadelic.stable(function MicrostateType(Type) {
  var _class, _temp;

  if (Type.Type) {
    return Type;
  }

  let Microstate = (_temp = _class = class Microstate extends Observable(Type) {
    constructor(value) {
      super(value);
      Object.defineProperties(this, funcadelic.map((slot, key) => {
        let relationship = slot instanceof Relationship ? slot : legacy(slot);
        return CachedProperty(key, self => {
          let _relationship$travers = relationship.traverse(new Edge(self, [key])),
              Type = _relationship$travers.Type,
              value = _relationship$travers.value;

          return mount(self, create(Type, value), key);
        });
      }, this));
      Object.defineProperty(this, Meta.symbol, {
        enumerable: false,
        configurable: true,
        value: new Meta(this, valueOf(value))
      });
    }

  }, _defineProperty(_class, "name", `Microstate<${Type.name}>`), _defineProperty(_class, "Type", Type), _temp);
  Object.defineProperties(Microstate.prototype, funcadelic.map(descriptor => {
    return {
      value(...args) {
        let result = descriptor.value.apply(sourceOf(this), args);
        let meta = metaOf(this);
        let previous = valueOf(meta.root);
        let next = set(meta.lens, valueOf(result), previous);

        if (meta.path.length === 0 && metaOf(result) != null) {
          return result;
        }

        if (next === previous) {
          return meta.root;
        } else {
          return create(meta.root.constructor, next);
        }
      }

    };
  }, funcadelic.append({
    set: {
      value: x => x
    }
  }, methodsOf(Type))));
  return Microstate;
});

function legacy(object) {
  let cell;
  let meta = metaOf(object);

  if (meta != null) {
    cell = {
      Type: object.constructor.Type,
      value: valueOf(object)
    };
  } else {
    cell = dsl.expand(object);
  }

  let _cell = cell,
      Type = _cell.Type;
  return relationship(cell.value).map(({
    value
  }) => ({
    Type,
    value
  }));
}

function hasOwnProperty(target, propertyName) {
  return Object.prototype.hasOwnProperty.call(target, propertyName);
}

class Literal {
  initialize(value) {
    if (value == null) {
      return this;
    }

    value = value.valueOf();

    switch (typeof value) {
      case "number":
        return create(Number, value);

      case "string":
        return create(String, value);

      case "boolean":
        return create(Boolean, value);

      default:
        if (Array.isArray(value)) {
          return create([Literal], value);
        } else {
          return create({
            Literal
          }, value);
        }

    }
  }

}

var literal = (value => create(Literal, value));

exports.Any = Any;
exports.ArrayType = ArrayType;
exports.BooleanType = BooleanType;
exports.NumberType = NumberType;
exports.ObjectType = ObjectType;
exports.Primitive = Primitive;
exports.Store = Identity;
exports.StringType = StringType;
exports.create = create;
exports.filter = filter;
exports.find = find;
exports.from = literal;
exports.map = map;
exports.metaOf = metaOf;
exports.reduce = reduce;
exports.relationship = relationship;
exports.valueOf = valueOf;
//# sourceMappingURL=microstates.cjs.js.map
